list, graph-box {
    border: 1px solid alpha(@theme_fg_color, 0.2);
}
list row {
    padding: 0;
}

list separator {
    background: alpha(@theme_fg_color, 0.2);
}

list row.max {
    background: alpha(@error_color, 0.3);
}

list row.max:hover {
    background: alpha(@error_color, 0.4);
}

list row.opened {
    background:alpha(@theme_fg_color, 0.2);
}

list row.opened:hover {
    background: alpha(@theme_fg_color, 0.3);
}

subprocess-list row {
    padding: 0;
    background: @theme_bg_color;
}

subprocess-list row:hover {
    background: alpha(@theme_fg_color, 0.3);
}

subprocess-list row.max {
    background: #fabbbb;
    border: 1px solid #faa5a2;
}

subprocess-list row.max:hover {
    background: #faa5a2;
    border: 1px solid #faa5a2;
}

button.graph-switcher {
    border-style: none;
    border-radius: 0;
}

rg-graph.big {
    background-size: 57px 57px;
    background-image: repeating-linear-gradient(0deg, alpha(@theme_fg_color, 0.2), alpha(@theme_fg_color, 0.2) 1px, transparent 1px, transparent 57px);
}

rg-graph.line {
    color: rgba(74,144,217,1);
}

rg-graph.stacked {
    color: rgba(74,144,217,0.325);
}

rg-graph.line_orange {
    color: rgba(255,153,0,1);
}

rg-graph.stacked_orange {
    color: rgba(255,153,0,0.353);
}

rg-graph.line_max {
    color: rgba(238,34,34,1);
}

rg-graph.stacked_max {
    color: rgba(238,34,34,0.227);
}

PieChart.used, ColorRectangle.used {
    color: #4a90d9;
}

PieChart.others, ColorRectangle.others {
    color:  @insensitive_fg_color;
}
PieChart.available, ColorRectangle.available {
    color: @theme_base_color;
}

ColorRectangle.system, StorageGraph.system {
    color: #2e3436;
}

ColorRectangle.applications, StorageGraph.applications {
    color: #4e9a06;
}

ColorRectangle.trash, StorageGraph.trash {
    color: @theme_fg_color;
}

ColorRectangle.user, StorageGraph.user {
    color: #ad7fa8;
}

ColorRectangle.available-storage, StorageGraph.available-storage {
    color: #d3d7cf;
}

row.folders, list.folders {
    color: #457fd3;
}

box.storage {
    background-color: #ffffff;
}

/* Speedometer */
.speedometer-outter, .speedometer-inner, .speedometer-content-area {
    border-radius: 50%;
}

.speedometer-outter {
    background-color: @theme_selected_bg_color;
}

.speedometer-inner {
    margin: -1px;
    background: linear-gradient(90deg, transparent 50%, @theme_base_color 50%),
                linear-gradient(90deg, @theme_base_color 50%, transparent 50%);
    transition: background 200ms ease-in-out;
}

.speedometer-content-area {
    background-color: @theme_bg_color;
    margin: 5px;
}

.tag {
    border-style: none;
    border-radius: 3px;
    padding: 3px 6px;
    color: @theme_base_color;
}

.tag.tag-root {
    background-color: red;
}

.tag.tag-system {
    background-color: orange;
}

.tag.tag-user {
    background: alpha(@theme_fg_color, 0.4);
}
